mod literals;
mod parser;
mod rtlil;

use nom::error::{convert_error, VerboseError};
use std::path::PathBuf;
use std::fs::File;
use std::io::prelude::*;
use structopt::StructOpt;

#[derive(Debug, StructOpt)]
#[structopt(name = "example", about = "An example of StructOpt usage.")]
struct Opt {
    /// Input file
    #[structopt(parse(from_os_str))]
    input: PathBuf,
}

// custom IResult type for verboseerror
pub type IResult<I, O, E = VerboseError<I>> = nom::IResult<I, O, E>;

fn main() {
    let opt = Opt::from_args();
    let mut infile = File::open(opt.input).expect("could not open file");
    let mut input = String::new();
    infile.read_to_string(&mut input).expect("error reading file");
    let input: &str = input.as_str();
    let parsed = parser::parse(&input);
    match parsed {
        Err(nom::Err::Error(err) | nom::Err::Failure(err)) => {
            print!("{}", convert_error(input, err))
        }
        Err(_) => (),
        Ok(res) => {
            println!("{:#?}", res);
            let lowered = crate::rtlil::lower_module(res.1);
            println!("{}", lowered);
        }
    }
}
